use opentelemetry_otlp::WithExportConfig;
use opentelemetry_sdk::Resource;
use ovlach_pdf::rocket_builder;
use rocket::launch;


use opentelemetry::trace::{TracerProvider as _, TraceError};
use opentelemetry_sdk::trace::{TracerProvider, Config};
use opentelemetry_stdout as stdout;
use tokio::net::unix::pipe;
use tracing::{error, span};
use tracing_subscriber::{layer::SubscriberExt, util::SubscriberInitExt};
use tracing_subscriber::Registry;


use opentelemetry::{
    metrics::MeterProvider as _,
    trace::{Tracer, TracerProvider as _},
    KeyValue,
};
use opentelemetry_sdk::{
    metrics::{PeriodicReader, MeterProvider},
    runtime,
};


fn init_meterprovider() -> MeterProvider {
    let exporter = opentelemetry_stdout::MetricsExporter::default();
    let reader = PeriodicReader::builder(exporter, runtime::Tokio).build();
    MeterProvider::builder().with_reader(reader).build()
}


fn init_tracer() -> Result<opentelemetry_sdk::trace::Tracer, TraceError> {
    opentelemetry_otlp::new_pipeline()
        .tracing()
        .with_exporter(
            opentelemetry_otlp::new_exporter()
                .tonic()
                .with_endpoint("http://localhost:4317"),
        )
        .with_trace_config(
            opentelemetry_sdk::trace::config().with_resource(Resource::new(vec![KeyValue::new(
                opentelemetry_semantic_conventions::resource::SERVICE_NAME,
                "ovlach_pdf",
            )])),
        )
        .install_batch(runtime::Tokio)
        //.install_simple()
    }

#[rocket::main]
async fn main() {
    let logger = opentelemetry_stdout::SpanExporter::default();
    let provider = TracerProvider::builder()
        .with_simple_exporter(logger)
        .build();

    let telemetry = tracing_opentelemetry::layer().with_tracer(
        provider.tracer("default")
    ).with_tracer(
        init_tracer().unwrap()
    );
    let meter_provider = init_meterprovider();
    Registry::default().with(telemetry).init();
    let _ = rocket_builder().launch().await;
}